@@ -7,6 +7,7 @@ from flask import request, logging, redirect
 from flask.ext import restful
 
 from pagador import MeioDePagamentoNaoEncontrado, MeioDePagamentoNaoDefinido
+from pagador.configuracao.cadastro import DadosInvalidos
 from pagador.configuracao.models import FormaDePagamento, FormaDePagamentoConfiguracao
 from pagador.envio.requisicao import EnviarPedido
 from pagador.extensao.base import PluginNaoEncontrado
@@ -27,13 +28,20 @@ class FormasPagamentoResource(restful.Resource):
     def get(self):
         conta_id = request.args.get('conta_id', None)
         plano_indice = request.args.get('plano_indice', 1)
+        soh_habilitados = request.args.get('soh_habilitados', "False") == "True"
         com_configuracao = False
         if conta_id:
             com_configuracao = True
             formas_pagamento = FormaDePagamento.configuracoes_ativas(conta_id=conta_id)
         else:
             formas_pagamento = FormaDePagamento.objects.filter(ativo=True)
-        return {"formas_pagamento": [forma.to_dict(com_configuracao=com_configuracao, plano_indice=plano_indice) for forma in formas_pagamento]}
+
+        formas_dict = []
+        for forma_pagamento in formas_pagamento:
+            forma_dict = forma_pagamento.to_dict(com_configuracao=com_configuracao, plano_indice=plano_indice, soh_habilitados=soh_habilitados)
+            if forma_dict:
+                formas_dict.append(forma_dict)
+        return {"formas_pagamento": formas_dict}
 
 
 class PagamentoBaseResource(restful.Resource):
@@ -92,7 +100,23 @@ class FormaPagamentoResource(PagamentoBaseResource):
         return self.retorno(pagamento_id)
 
 
-class PagamentoConfiguracaoBaseResource(PagamentoBaseResource):
+class PagamentoConfiguracaoBaseResource(restful.Resource):
+
+    def __init__(self):
+        self._dados = None
+        self._pagamento_configuracao = None
+        self.pagamento_id = None
+        self.conta_id = None
+
+    @property
+    def dados(self):
+        if not self._dados:
+            self._dados = {}
+            for arg in request.args:
+                self._dados[arg] = request.args[arg]
+            for arg in request.form:
+                self._dados[arg] = request.form[arg]
+        return self._dados
 
     @property
     def chave(self):
@@ -104,19 +128,18 @@ class PagamentoConfiguracaoBaseResource(PagamentoBaseResource):
 
     @property
     def pagamento_configuracao(self):
-        if self.conta_id:
-            return FormaDePagamentoConfiguracao.objects.get(
-                conta_id=self.conta_id,
-                forma_pagamento_id=self.pagamento_id
-            )
-        return None
+        if self._pagamento_configuracao is None:
+            if self.conta_id:
+                self._pagamento_configuracao = FormaDePagamentoConfiguracao.objects.get(
+                    conta_id=self.conta_id,
+                    forma_pagamento_id=self.pagamento_id
+                )
+        return self._pagamento_configuracao
 
     @autenticacao.requerido
     def get(self, pagamento_id, conta_id):
         self.pagamento_id = pagamento_id
         self.conta_id = conta_id
-        if self.retorno_tem_erro:
-            return self.retorno()
         try:
             return {self.chave: self.valor}
         except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
@@ -130,12 +153,10 @@ class PagamentoConfiguracaoBaseResource(PagamentoBaseResource):
             return {"erro": u"Erro de instalação: {}".format(ex)}, 500
 
 
-class FormaPagamentoConfiguracaoResource(PagamentoBaseResource):
+class FormaPagamentoConfiguracaoResource(restful.Resource):
     @autenticacao.requerido
     def get(self, pagamento_id, conta_id):
         self.pagamento_id = pagamento_id
-        if self.retorno_tem_erro:
-            return self.retorno()
         try:
             pagamento_configuracao, criado = FormaDePagamentoConfiguracao.objects.get_or_create(
                 conta_id=conta_id,
@@ -163,20 +184,19 @@ class FormaPagamentoConfiguracaoResource(PagamentoBaseResource):
             return {"erro": u"pagador: {}".format(ex)}, 500
         except IntegrityError, ex:
             return {"erro": u"Erro no banco de dados: {}".format(ex)}, 500
+        except DadosInvalidos, ex:
+            return {"validacao": u"{}".format(ex), "erros": ex.erros}, 400
 
 
 #FIXME: Refatorar isso, pois tá repetido
-class FormaPagamentoConfiguracaoComplementoResource(PagamentoBaseResource):
+class FormaPagamentoConfiguracaoComplementoResource(PagamentoConfiguracaoBaseResource):
     #FIXME: ALERTA!!! ALERTA!!! SEM TESTES!!! ALERTA!!! ALERTA!!!
     @autenticacao.requerido
     def post(self, pagamento_id, conta_id):
-        dados = request.get_json()
+        self.pagamento_id = pagamento_id
+        self.conta_id = conta_id
         try:
-            pagamento_configuracao = FormaDePagamentoConfiguracao.objects.get(
-                conta_id=conta_id,
-                forma_pagamento_id=pagamento_id,
-            )
-            resultado = pagamento_configuracao.salvar_complemento(dados)
+            resultado = self.pagamento_configuracao.complemento(self.dados)
             return {"message": resultado["content"]}, resultado["status"]
         except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
             return {"erro": u"pagador: {}".format(ex)}, 500
@@ -252,22 +272,13 @@ class FormaPagamentoInstalacaoResource(PagamentoConfiguracaoBaseResource):
 
 
 #FIXME: ALERTA!!! ALERTA!!! SEM TESTES!!! ALERTA!!! ALERTA!!!
-class FormaPagamentoConfiguracaoEnviarResource(PagamentoBaseResource):
+class FormaPagamentoConfiguracaoEnviarResource(PagamentoConfiguracaoBaseResource):
     @autenticacao.requerido
     def post(self, pagamento_id, conta_id, pedido_numero):
-        dados = {}
-        if request.data:
-            dados = json.loads(request.data)
         self.pagamento_id = pagamento_id
-        pagamento = self.forma_de_pagamento_to_dict
-        if self.retorno_tem_erro:
-            return self.retorno()
+        self.conta_id = conta_id
         try:
-            pagamento_configuracao = FormaDePagamentoConfiguracao.objects.get(
-                conta_id=conta_id,
-                forma_pagamento_id=pagamento["id"]
-            )
-            enviar_pedido = EnviarPedido(pagamento["codigo"], pedido_numero, dados, conta_id, pagamento_configuracao)
+            enviar_pedido = EnviarPedido(pedido_numero, self.dados, conta_id, self.pagamento_configuracao)
             resultado = enviar_pedido.enviar()
             return resultado
         except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
@@ -286,11 +297,9 @@ class FormaPagamentoRetornoResource(PagamentoConfiguracaoBaseResource):
     def get(self, pagamento_id, conta_id=None, tipo="retorno"):
         self.pagamento_id = pagamento_id
         self.conta_id = conta_id
-        if self.retorno_tem_erro:
-            return self.retorno()
         try:
             self.dados["conta_id"] = conta_id
-            registro = RegistrarPagamento(self.forma_de_pagamento.codigo, self.dados, tipo, self.pagamento_configuracao)
+            registro = RegistrarPagamento(self.dados, tipo, self.pagamento_configuracao)
             resultado = registro.processar()
             if "redirect" in resultado:
                 return redirect(resultado["redirect"], code=301)
