@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 import datetime
 
-from repositories.pedido.models import PedidoVendaSituacao, PedidoVendaFormaPagamento
+from repositories.pedido.models import PedidoVendaSituacao, PedidoVendaFormaPagamento, PedidoVenda
 
 
 class SituacaoPedido(PedidoVendaSituacao):
@@ -12,3 +12,8 @@ class SituacaoPedido(PedidoVendaSituacao):
 class PedidoPagamento(PedidoVendaFormaPagamento):
     class Meta:
         proxy = True
+
+
+class Pedido(PedidoVenda):
+    class Meta:
+        proxy = True
